<?php
/**
 * Application
 * @package     Me
 * @copyright   Copyright (C) 2011 MicroEngine Inc.
 */
class Me_Application
{
    /** デフォルトコントローラ */
    const DEFAULT_CONTROLLER = 'Main';
    /** デフォルトアクション */
    const DEFAULT_ACTION = 'index';
    /**
     * 認証済み
     * @var boolean
     */
    public $authenticated = false;
    /**
     * コントローラリスト
     * @var array
     */
    private $controllers = array(
        'Main' => 'Main_Controller',
        'Metag' => 'Metag_Controller',
        'Page' => 'Page_Controller',
        'System' => 'System_Controller',
        'License' => 'License_Controller',
        'Template' => 'Template_Controller',
        'User' => 'User_Controller',
        'Help' => 'Help_Controller',
        'Install' => 'Install_Controller',
    );

    /**
     * コンストラクタ
     */
    public function __construct()
    {

    }

    /**
     * run
     * @param string $force_controller
     */
    public function run($force_controller = null)
    {
        try {
            // routing
            list($class, $action) = $this->routing($force_controller);

            // Controllerオブジェクト作成
            require_once('controllers/' . $class . '.php');
            $ctrl = new $class($action, $this);

            // Execute
            $ctrl->execute();
        } catch (Me_Exception $e) {
            header('X-MicroEngine-Error: ' . $e->getCode(), true, 500);
            $e->output($this->authenticated);
            exit;
        } catch (Exception $e) {
            echo $e->getMessage();
            exit;
        }
    }

    /**
     * routing
     * @return array
     */
    private function routing()
    {
        // Controllerとaction取得
        $controller = self::DEFAULT_CONTROLLER;
        $action = self::DEFAULT_ACTION;

        // path info を取得
        // REQUEST_URI から SCRIPT_NAME 部分を除外
        $path_info = substr($_SERVER['REQUEST_URI'], strlen($_SERVER['SCRIPT_NAME']));
        // ? があれば、？以降を除外
        if (strpos($path_info, '?') !== false) {
            $path_info = substr($path_info, 0, strpos($path_info, '?'));
        }

        // $path_info を配列に分解して、パスを確認する
        $path_info_array = explode('/', $path_info);
        // $path_infoの階層の深さが合わない場合は、/Main/indexにリダイレクトする
        if (count($path_info_array) !== 3) {
            Me_Util::redirect(str_repeat('../', count($path_info_array) - 1) . 'index.php/Main/index');
        }
        if ($path_info_array[1] !== '') {
            $controller = $path_info_array[1];
        }
        if ($path_info_array[2] !== '') {
            $action = $path_info_array[2];
        }

        // スキーマ情報確認
        $meinfo_dao = new Meinfo_Dao();
        $meinfo = $meinfo_dao->select(1, 'Meinfo', true);
        if ($meinfo === false) {
            // スキーマ情報を確認できない場合はエラー
            throw new Me_Exception(6);
        } else if ($meinfo->schema_version === '0') {
            // 未インストール状態
            // Install以外のコントローラーが呼ばれた場合は、Installにリダイレクト
            if ($controller !== 'Install') {
                // インストーラーにリダイレクトする
                Me_Util::redirect('../Install/index');
            }
        } else if ($meinfo->schema_version !== ME_SCHEMA_VERSION) {
            // スクリプトとスキーマのバージョンが違う場合はエラーとする。
            throw new Me_Exception(2);
        } else {
            // インストール済みで、スキーマバージョンも一致
            if ($controller === 'Install') {
                // インストール後に、Installコントローラーは実行できな
                throw new Me_Exception(5);
            }
        }

        // コントローラーの存在確認
        if (isset($this->controllers[$controller])) {
            $class = $this->controllers[$controller];
        } else {
            throw new Me_Exception(3, $controller);
        }
        return array($class, $action);
    }
}