<?php
/**
 * License Util
 * @package     License
 * @copyright   Copyright (C) 2011 MicroEngine Inc.
 */
class License_Util
{
    /* sessionキー名 */
    const SESSION_KEY = '_license';
    /* 試用期間中 */
    const STATUS_TRIAL = 'trial';
    /* 有効 */
    const STATUS_VALID = 'valid';
    /* 無効 */
    const STATUS_INVALID = 'invalid';
    /* 試用期間日数 */
    const TRAIAL_DAYS = 30;
    /* 1日の秒数 */
    const DAY_SECONDS = 86400;

    /**
     * @var License
     */
    static private $license;

    /**
     * ライセンス認証を実行する
     */
    static public function auth()
    {
        /* @var $license License */
        $license = null;
        if (isset($_SESSION[self::SESSION_KEY])) {
            $license = $_SESSION[self::SESSION_KEY];
        }

        // 有効なライセンスがあるかどうか
        if ($license === null || $license->status !== self::STATUS_VALID) {
            // インストール日時を取得
            $meinfo_dao = new Meinfo_Dao();
            $meinfo = $meinfo_dao->select(1, 'Meinfo', true);

            // ライセンス情報を取得
            $logic = new License_Logic();
            $license = $logic->select_license_key(1, 'License');
            $license->install_date = $meinfo->install_date;
            // ライセンス認証を実行する
            if ($logic->authentication($license->license_id, $license->license_key)) {
                // 有効なライセンスが登録されている
                $license->status = self::STATUS_VALID;
            } else {
                // 試用期間の確認
                $now = time();
                $period = $meinfo->install_date + (self::TRAIAL_DAYS * self::DAY_SECONDS);
                // インストール日時より1日前から30日以内だと試用期間と判定する。
                if (($meinfo->install_date - self::DAY_SECONDS) < $now && $now <= $period) {
                    // 試用期間中
                    $license->status = self::STATUS_TRIAL;
                } else {
                    // 試用期間切れ
                    $license->status = self::STATUS_INVALID;
                }
            }
        }

        $_SESSION[self::SESSION_KEY] = $license;
        self::$license = $license;
    }

    /**
     * ライセンスが登録されているか、試用期間中であることを確認する。
     * 試用期間が過ぎていればエラーとする。
     */
    static function check_valid()
    {
        $license = self::$license;
        if ($license === null) {
            throw new Me_Exception(700);
        }
        if ($license->status === self::STATUS_INVALID) {
            throw new Me_Exception(701);
        }
    }

    /**
     * ライセンス情報取得
     * @return string $message
     */
    static function get_message()
    {
        $message = '';
        $license = self::$license;
        if ($license === null) {
            $message = 'ライセンスが登録されていません。';
            return $message;
        }
        switch ($license->status) {
            case self::STATUS_TRIAL:
                // 試用期間の残り日数を計算する。
                // インストール日時を取得
                $period = $license->install_date + (self::TRAIAL_DAYS * self::DAY_SECONDS);
                $last = $period - time();
                $message = '試用期間は残り ' . gmdate('z日H時間i分s秒', $last) . ' です。';
                break;
            case self::STATUS_INVALID:
                $message = '試用期間が過ぎています。ライセンスを登録してください。';
                break;
            default:
                break;
        }
        return $message;
    }
}
