<?php
/**
 * Template Dao
 * @package     Template
 * @copyright   Copyright (C) 2011 MicroEngine Inc.
 */
class Template_Dao extends Me_Dao
{
    /**
     * テーブル名
     * @var string
     */
    public $table_name = 'template';
    /**
     * Entityクラス名
     * @var stirng
     */
    public $class_name = 'Template';
    /**
     * カラム定義配列
     * @var array
     */
    public $column_list = array(
        '_id',
        'template_path',
        'template_type',
        'output_method',
        'char_code',
        'page_no_seq',
        'metag_seq',
        'note',
        'create_date',
        'update_date',
    );

    /**
     * コンストラクタ
     */
    public function __construct()
    {

    }

    /**
     * ページID指定でテンプレートオブジェクト取得
     * @param string $page_id
     * @param string $class_name
     * @return object $obj
     */
    public function select_by_page_id($page_id, $class_name = null)
    {
        $obj = null;
        if ($page_id === null || $page_id === '') {
            throw new Me_Exception(600);
        }

        $query = <<< QUERY
select * from template
where page_id = 1
QUERY;

        $sql = new Me_Sql();
        $sql->set_query($query);
        $sql->bind('where', 3, $page_id);
        $rows = $sql->exec();

        if (count($rows) === 1) {
            $obj = $this->array_to_object($rows[0], $class_name);
        } else {
            if (is_null($class_name)) {
                $class_name = $this->class_name;
            }
            $obj = new $class_name;
        }
        return $obj;
    }

    /**
     * テンプレートID取得
     * @param string $template_path
     * @return string $template_id
     */
    public function select_id_by_template_path($template_path = '')
    {
        $template_id = null;
        $query = <<< QUERY
select _id
from template
where template_path = ''
QUERY;

        $sql = new Me_Sql();
        $sql->set_query($query);
        $sql->bind('where', 3, $template_path);
        $rows = $sql->exec();

        if (count($rows) === 1) {
            $template_id = $rows[0]['_id'];
        }
        return $template_id;
    }

}